;##################################################################
; Font Menu
;##################################################################

#include "FontMenu.Inc"

;#include "\RadASM\GoAsm\inc\debug.a"
;DBGWIN_DEBUG_ON = 1 ; use this to enable Spy/StopSpy/DbgDump and TrapException
;DBGWIN_SHOWONLYERRORS = 1 ; setting this to 1 will suppress PrintError if no error exists 

.CONST
	TOPMSG				equ					2048
	IDD_DLG1			equ					1000
.DATA
	szFontMenuLabel	DB			"Fonts",0

	ALIGN 4
	icc	 			DQ			3FFF00000008h

	MsgTable		DD			TOPMSG DUP (?)
	hInstance		DD 			?
	hMNBrush		DD			?
	hMenu			DD			?
	hFontMenu		DD			?
	hDlg			DD			?

	nItemNum		DD			?
	
	hChkIML			DD			?
	
	CurrentFont		DD			0

.CODE
BuildMessageTable:
	uses edi,ecx,eax
		; Set the default proc for all messages to WM_USER+1023
		mov edi,OFFSET MsgTable
		mov ecx,TOPMSG-1
		mov eax,OFFSET DEFPROC
		rep stosd
		; Set the offsets of the messages to process
		mov [MsgTable+(WM_INITDIALOG *4)],OFFSET WMINITDIALOG
		mov [MsgTable+(WM_COMMAND *4)],OFFSET WMCOMMAND
		mov [MsgTable+(WM_CLOSE *4)],OFFSET WMCLOSE
		mov [MsgTable+(WM_MEASUREITEM *4)],OFFSET WMMEASUREITEM
		mov [MsgTable+(WM_DRAWITEM *4)],OFFSET WMDRAWITEM
		ret
	endu

Start:
	; I use a message table to handle messages
	; this will build the table
	call BuildMessageTable
	invoke GetModuleHandle, 0
	mov [hInstance],eax
	; Common Controls is needed for the imagelist functions
	invoke InitCommonControlsEx,OFFSET icc
	; Create a brush to use as the Selection color
	invoke CreateSolidBrush,00FF8080h
	mov [hMNBrush],eax
	; load the check marks image list from the resource section of our app
	invoke ImageList_LoadImage,[hInstance],5000,11,10,0FFFFFFh,IMAGE_BITMAP,LR_CREATEDIBSECTION
	mov [hChkIML],eax
	invoke DialogBoxParam,[hInstance],IDD_DLG1,0,ADDR DlgProc,0
	; Clean up the resources and shut down
	invoke DeleteObject,[hMNBrush]
	invoke DestroyMenu,[hFontMenu]
	invoke DestroyMenu,[hMenu]
	invoke ImageList_Destroy,[hChkIML]
	invoke ExitProcess,0

DlgProc FRAME hwnd,uMsg,wParam,lParam
	uses edi,ebx,esi
	LOCAL lf		:LOGFONT
	LOCAL rect		:RECT
	LOCAL lbr		:LOGBRUSH
	LOCAL mii		:MENUITEMINFO

	mov eax,[uMsg]
	cmp eax,TOPMSG-1
	ja >DEFPROC
	jmp [MsgTable+eax*4]

	WMINITDIALOG:
		mov eax,[hwnd]
		mov [hDlg],eax
		; Create a menu bar for our dialog
		invoke CreateMenu
		mov [hMenu],eax
		; Assign the menu to the dialog
		invoke SetMenu,[hwnd],eax
		; Create a submenu for the fonts
		invoke CreatePopupMenu
		mov [hFontMenu],eax
		; Enumerate the fonts and add them to the menu, enumerate for ANSI_CHARSET
		invoke BuildFontMenu,eax,ANSI_CHARSET
		; Add our font menu to the menu bar
		invoke InsertMenu,[hMenu],0,MF_BYPOSITION + MF_POPUP + MF_STRING,[hFontMenu],OFFSET szFontMenuLabel
		; Redraw the menu bar to display the font menu
		invoke DrawMenuBar,[hwnd]
		jmp >>.EXIT

	WMDRAWITEM:
		; If wParam is 0 then Windows is asking to draw a menu item
		cmp D[wParam],0
		jne >DEFPROC
			; lParam hold a pointer to the DRAWITEMSTRUCT structure
			; We will use this structure to give us references for drawing
			; Just to make sure this is a menu we will verify the control type
			mov edi,[lParam]
			mov eax,[edi+DRAWITEMSTRUCT.CtlType]
			test eax,ODT_MENU
			je >DEFPROC
				; itemAction tells us what type of drawing is taking place
				; for menus we need to draw for ODA_DRAWENTIRE and ODA_SELECT
				; for our purpose we will not distinguish between itemActions
				; Get the item state (either selected or not)
				mov eax,[edi+DRAWITEMSTRUCT.itemState]
				test eax,ODS_SELECTED
				je >>S1
					; This menu item is selected so we will use the menu
					; brush, first we need the color of the brush for the
					; font background color
					mov ebx,0FFFFFFh
					mov esi,[hMNBrush]
					invoke GetObject,esi,SIZEOF LOGBRUSH,OFFSET lbr
					push [lbr.lbColor]
					jmp >S2
				S1:
					; This menu item is not selected so we will create a
					; brush from COLOR_BUTTONFACE
					mov ebx,0
					invoke GetSysColor,COLOR_BTNFACE
					push eax
					invoke CreateSolidBrush,eax
					mov esi,eax
				S2:
					pop eax
					invoke SetBkColor,[edi+DRAWITEMSTRUCT.hdc],eax
					push eax ; push the old background color

					; Set the text color to WHITE
					invoke SetTextColor,[edi+DRAWITEMSTRUCT.hdc],ebx
					push eax ; push the old text color

					; The itemData member has our font handle we need to extract the
					; name of the font to draw it
					invoke GetObject,[edi+DRAWITEMSTRUCT.itemData],SIZEOF LOGFONT,OFFSET lf

					; Select the font into the menu's DC so any text will be drawn with that font
					invoke SelectObject,[edi+DRAWITEMSTRUCT.hdc],[edi+DRAWITEMSTRUCT.itemData]
					push eax ; Push the old object onto the stack

					; Calculate the offset of the bounding RECT structure for the item
					; we are drawing
					mov ebx,edi
					add ebx,DRAWITEMSTRUCT.rcItem

					; Fill the bounding rectangle with the selection brush
					invoke FillRect,[edi+DRAWITEMSTRUCT.hdc],ebx,esi
					
					; Get the indent we will need to draw the text and add it to the left
					; side of the rect, we don't have to return this structure so we will
					; write directly to it
					invoke GetMenuCheckMarkDimensions
					and eax,0FFFFh ; The low order word contains the width
					add [edi+DRAWITEMSTRUCT.rcItem.left],eax

					invoke lstrlen,OFFSET lf.lfFaceName
					or eax,eax
					jnz >
						invoke lstrcpy,OFFSET lf.lfFaceName,"Custom font"
					:
					; Draw the text and restore all of the objects and colors
					invoke DrawText,[edi+DRAWITEMSTRUCT.hdc],OFFSET lf.lfFaceName,-1,ebx,DT_LEFT + DT_VCENTER + DT_SINGLELINE
					pop eax
					invoke SelectObject,[edi+DRAWITEMSTRUCT.hdc],eax
					pop eax
					invoke SetTextColor,[edi+DRAWITEMSTRUCT.hdc],eax
					pop eax
					invoke SetBkColor,[edi+DRAWITEMSTRUCT.hdc],eax

					; Draw the check mark if necessary
					mov ebx,edi
					add ebx,DRAWITEMSTRUCT.rcItem
					invoke GetMenuCheckMarkDimensions
					and eax,0FFFFh ; The low order word contains the width
					sub [edi+DRAWITEMSTRUCT.rcItem.left],eax

					; Center the image vertically we need to know the height of the checkmark
					invoke ImageList_GetIconSize,[hChkIML],OFFSET rect.left,OFFSET rect.top
					or eax,eax
					jne >
						; No image list or an error so don't draw it
						jmp >S3
					:
					mov ecx,[edi+DRAWITEMSTRUCT.rcItem.bottom]
					sub ecx,[edi+DRAWITEMSTRUCT.rcItem.top]
					sub ecx,[rect.left]
					shr ecx,1
					add [edi+DRAWITEMSTRUCT.rcItem.top],ecx

					; Is the menu item checked ??
					xor eax,eax
					test D[edi+DRAWITEMSTRUCT.itemState],MF_CHECKED
					je >
						inc eax
					:
					invoke ImageList_Draw,[hChkIML],eax,[edi+DRAWITEMSTRUCT.hdc],[ebx+RECT.left],[ebx+RECT.top],ILD_NORMAL

					mov eax,[edi+DRAWITEMSTRUCT.itemState]
					test eax,ODS_SELECTED
					jne >S3
						; Destroy the brush if it was a normal menu item
						invoke DeleteObject,esi
				S3:
					xor eax,eax
					inc eax
					ret

	WMMEASUREITEM:
		; The WM_MEASUREITEM message is sent for owner drawn controls
		; to tell Windows how big it will be
		; lParam hold a pointer to the MEASUREITEMSTRUCT structure
		; We will return this structure with the measurements of our control
		mov edi,[lParam]
		; If wParam is 0 then Windows is asking to measure a menu item
		cmp D[wParam],0
		jne >DEFPROC
			; Just to make sure this is a menu we will verify the control type
			mov eax,[edi+MEASUREITEMSTRUCT.CtlType]
			test eax,ODT_MENU
			je >DEFPROC
				; To measure the size of the text we must select it into a DC
				; We create a memory DC for this
				invoke GetDC,NULL
				mov ebx,eax
				invoke CreateCompatibleDC,ebx
				mov esi,eax
				invoke SelectObject,esi,[edi+MEASUREITEMSTRUCT.itemData]
				; Save the old object in the DC on the stack
				push eax
				; We stored the font handle in the lParam of the menu item
				; it is passed in MEASUREITEMSTRUCT.itemData, we will use that
				; to read the full LOGFONT structure to get the font name
				invoke GetObject,[edi+MEASUREITEMSTRUCT.itemData],SIZEOF LOGFONT,OFFSET lf
				; we need the length of the name
				invoke lstrlen,OFFSET lf.lfFaceName
				; Have Windows calculate the width and height of the string
				; This function takes a POINT structure but we will just reuse the rect
				; structure used elsewhere to limit the DlgProc stack size
				invoke GetTextExtentPoint32,esi,OFFSET lf.lfFaceName,eax,OFFSET rect
				; Pop the old object in the DC off the stack and select it back into the DC
				pop ecx
				invoke SelectObject,esi,ecx
				; Delete the memory DC and release the screen DC
				invoke DeleteDC,esi
				invoke ReleaseDC,NULL,ebx
				; We need to know how much space to leave for a check mark
				; This information is obtained from GetMenuCheckMarkDimensions
				; add it to the left side of the rectangle (width)
				invoke GetMenuCheckMarkDimensions
				mov edx,eax
				shr edx,16 ; The high order word contains the height (EDX)
				and eax,0FFFFh ; The low order word contains the width (EAX)
				add eax,[rect.left]
				; Set the width of this item
				mov D[edi+MEASUREITEMSTRUCT.itemWidth],eax
				; if our height is below the height of a checkmark we 
				; set it to that value
				add edx,2 ; make sure we leave a pixel on the top and bottom
				mov ecx,[rect.top]
				cmp ecx,edx
				ja >
					mov ecx,edx
				:
				mov D[edi+MEASUREITEMSTRUCT.itemHeight],ecx
				xor eax,eax
				inc eax
				ret

	WMCOMMAND:
		; Get the command ID
		xor eax,eax
		mov ax,W[wParam]
		; Set up the MENUITEMINFO structure
		mov D[mii.cbSize],SIZEOF MENUITEMINFO
		mov D[mii.fMask],MIIM_DATA
		mov D[mii.dwItemData],0
		; Extract the information from this menu item
		invoke GetMenuItemInfo,[hMenu],eax,MF_BYCOMMAND,OFFSET mii
		; Make sure that it has a font handle in dwItemData
		cmp D[mii.dwItemData],0
		je >>.EXIT
			; Extract the LOGFONT information for this menu item
			; We will use that as a starting point for creating a new
			; font for use in the edit control
			invoke GetObject,[mii.dwItemData],SIZEOF LOGFONT,OFFSET lf
			; Modify the LOGFONT structure based on the users choices
			; do not modify the lf.lfFaceName or lf.lfPitchAndFamily members
			mov eax,-15 ; Display height of font
			mov [lf.lfHeight],eax
			mov D[lf.lfWidth],0
			; Create a new font for the edit
			invoke CreateFontIndirect,OFFSET lf
			mov ebx,eax
			; Get the old font and delete it
			; if there was no user selected font it will be a
			; system defined font (Stock Object) and it does no
			; harm to delete it anyway
			invoke SendDlgItemMessage,[hwnd],1001,WM_GETFONT,0,0
			invoke DeleteObject,eax ; destroy the old display font
			; Set the new font for the edit, redraw must be set to TRUE
			invoke SendDlgItemMessage,[hwnd],1001,WM_SETFONT,ebx,TRUE
			; Reset the checkmark on the old font (ID stored in [CurrentFont])
			invoke CheckMenuItem,[hMenu],[CurrentFont],MF_BYCOMMAND + MF_UNCHECKED
			; Set the checkmark in the new font selection and update [CurrentFont]
			xor eax,eax
			mov ax,W[wParam]
			mov [CurrentFont],eax
			invoke CheckMenuItem,[hMenu],eax,MF_BYCOMMAND + MF_CHECKED

		jmp >>.EXIT

	WMCLOSE:
		; To be good WinCitizens we must clean up before we leave
		; Step through the font menu and extract the font handles
		; and delete them.
		mov D[mii.cbSize],SIZEOF MENUITEMINFO
		mov D[mii.fMask],MIIM_DATA
		mov ebx,5001
		invoke GetMenuItemInfo,[hMenu],ebx,MF_BYCOMMAND,OFFSET mii
		jmp >W2
		W1:
			invoke DeleteObject,[mii.dwItemData]
			mov D[mii.dwItemData],0
			inc ebx
			invoke GetMenuItemInfo,[hMenu],ebx,MF_BYCOMMAND,OFFSET mii
		W2:
		cmp D[mii.dwItemData],0
		jne <W1
		INVOKE EndDialog,[hwnd],0

	DEFPROC:
		mov eax,FALSE
		ret

	.EXIT

	mov eax, TRUE
	ret
ENDF

BuildFontMenu FRAME SubMenuHandle,CharSet
	uses ebx,esi
	LOCAL lf	:LOGFONT

	; To enumerate the fonts we must suuply the information in
	; a LOGFONT structure to control the enumeration. The API will
	; read the lfCharSet, lfFaceName and lfPitchAndFamily members of
	; the structure. Always set the lfPitchAndFamily member to 0. If
	; you wish to enumerate all families set lfFaceName to 0. It is
	; good idea to supply a character set or you may get multiple
	; instances of the same font (one for each character set)
	; For the purposes of the menu we pass the menu handle of the Font menu
	; to the enumerator in lParam, this let's it add the menu items directly
	; as part of the enumeration
	mov D[nItemNum],6001
	mov eax,[CharSet]
	mov B[lf.lfCharSet],al
	mov B[lf.lfFaceName],0
	mov B[lf.lfPitchAndFamily],0 
	invoke GetDlgItem,[hDlg],1001
	mov esi,eax
	invoke GetDC,esi
	mov ebx,eax
	invoke EnumFontFamiliesEx,ebx,ADDR lf,OFFSET EnumFontFamExProc,[SubMenuHandle],0
	invoke ReleaseDC,esi,ebx
	RET
ENDF

EnumFontFamExProc FRAME lpelfe,lpntme,FontType,lParam
	uses ebx

	; Get the data from the font and create a font of a fixed height
	; for the menu to display
	mov ebx,[lpelfe]
	add ebx,ENUMLOGFONTEX.elfLogFont
	mov D[ebx+LOGFONT.lfHeight],-13
	mov D[ebx+LOGFONT.lfWidth],0
	mov D[ebx+LOGFONT.lfWeight],FW_NORMAL
	invoke CreateFontIndirect,ebx
	; Add this font to the menu
	invoke AppendMenu,[lParam],MF_ENABLED + MF_OWNERDRAW,[nItemNum],eax
	inc D[nItemNum]
	
	mov eax,TRUE
	RET
ENDF
